package com.persagy.dmp.middleware.controller;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.node.JsonNodeFactory;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.dmp.basic.model.QueryCriteria;
import com.persagy.dmp.middleware.constant.enums.RelationTypeEnum;
import com.persagy.dmp.middleware.entity.dto.CommonRequestDTO;
import com.persagy.dmp.middleware.entity.dto.InstanceUrlParam;
import com.persagy.dmp.middleware.entity.response.CommonResult;
import com.persagy.dmp.middleware.service.IBaseService;
import com.persagy.dmp.middleware.service.IFloorService;
import com.persagy.dmp.middleware.utils.EntityTransferUtil;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.ws.rs.QueryParam;

/**
 * @ClassName FloorController
 * @Description: 楼层
 * @Author linhuili
 * @Date 2021/9/9 10:24
 * @Version V1.0
 **/
@RestController
@RequestMapping("floor")
public class FloorController {

    @Autowired
    private IFloorService floorService;

    @Autowired
    private IBaseService baseService;

    /**
     * 查询建筑下的楼层列表
     * @param projectId
     * @param request
     * @return
     * @throws Exception
     */
    @PostMapping("/queryList")
    public CommonResult queryList(@QueryParam("groupCode") String groupCode,@QueryParam("projectId") String projectId,
                              @RequestBody CommonRequestDTO request) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, null);
        // 非空校验
        if(StringUtils.isEmpty(request.getBuildId())){
            throw new Exception("建筑ID不能为空");
        }
        // 条件转换
        QueryCriteria queryCriteria = EntityTransferUtil.transDmpRelationCriteria(RelationTypeEnum.ARCHSUBSET_BD2FL.getGraphCode(), RelationTypeEnum.ARCHSUBSET_BD2FL.getRelCode(), request.getBuildId(), 0);
        // 查询建筑下的楼层列表
        CommonResult response = baseService.doQuery(context,queryCriteria ,null);
        return  response;
    }


    /**
     * 统计楼层下各信息
     * @param projectId
     * @param request
     * @return
     * @throws Exception
     */
    @PostMapping("/count")
    public CommonResult count(@QueryParam("groupCode") String groupCode,@QueryParam("projectId") String projectId,
                              @RequestBody CommonRequestDTO request) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, null);
        // 非空校验
        if(StringUtils.isEmpty(request.getBuildId())){
            throw new Exception("建筑ID不能为空");
        }
        // 统计楼层下各信息
        CommonResult response = floorService.count(context,request);
        return  response;
    }

    /**
     *  查询/统计楼层下的设备，空间，阀门
     * @param projectId
     * @param request
     * @return
     * @throws Exception
     */
    @PostMapping("/query")
    public CommonResult query(@QueryParam("groupCode") String groupCode,@QueryParam("projectId") String projectId,
                              @RequestBody CommonRequestDTO request) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, null);
        // 非空校验
        if(StringUtils.isEmpty(request.getBuildId())){
            throw new Exception("建筑ID不能为空");
        }
        //查询/统计楼层下的设备空间
        CommonResult response = floorService.query(context,request);
        return  response;
    }


}
