package com.persagy.dmp.middleware.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.JsonNodeFactory;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.dmp.basic.model.QueryCriteria;
import com.persagy.dmp.define.entity.ObjectInfoDefine;
import com.persagy.dmp.digital.entity.ObjectDigital;
import com.persagy.dmp.middleware.constant.CommonConsant;
import com.persagy.dmp.middleware.constant.enums.RelationTypeEnum;
import com.persagy.dmp.middleware.dao.DefineTypeRelMapper;
import com.persagy.dmp.middleware.entity.dto.CommonRequestDTO;
import com.persagy.dmp.middleware.entity.dto.InstanceUrlParam;
import com.persagy.dmp.middleware.entity.response.CommonResult;
import com.persagy.dmp.middleware.entity.vo.DefineTypeRel;
import com.persagy.dmp.middleware.service.IBaseService;
import com.persagy.dmp.middleware.service.IFloorService;
import com.persagy.dmp.middleware.utils.EntityTransferUtil;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.sql.Wrapper;
import java.util.*;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

/**
 * @ClassName FloorServiceImpl
 * @Description: 楼层
 * @Author linhuili
 * @Date 2021/9/9 10:24
 * @Version V1.0
 **/
@Service
public class FloorServiceImpl implements IFloorService {

    @Autowired
    private IBaseService baseService;

    @Autowired
    private DefineTypeRelMapper defineTypeRelMapper;

    /**
     * 统计楼层下各信息
     * @param context
     * @param requestDTO
     * @return
     */
    @Override
    public CommonResult count(InstanceUrlParam context, CommonRequestDTO requestDTO) {
        ArrayNode result = JsonNodeFactory.instance.arrayNode();

        //查询楼层信息
        List<JSONObject> floorList = queryFloorList(context, requestDTO);
        if(CollUtil.isEmpty(floorList)){
            return CommonResult.success(result);
        }

       // Map<String, JSONObject> floorMap = floorList.stream().collect(Collectors.toMap(entry -> entry.getString("id"),entry->entry, (k1, k2) -> k1));
        ObjectNode content = JsonNodeFactory.instance.objectNode();
        //统计各楼层面积
        Long buildArea = getFloorArea(floorList, CommonConsant.FLOOR_AREA);
        //统计各楼层租赁面积
        Long leaseArea = getFloorArea(floorList, CommonConsant.FLOOR_LEASE_AREA);

        content.put("buildArea",buildArea);
        content.put("leaseArea",leaseArea);
        result.add(content);
        return CommonResult.success(content);
    }

    /**
     * 查询楼层下的设备，空间，阀门
     * @param context
     * @param requestDTO
     * @return
     */
    @Override
    public CommonResult query(InstanceUrlParam context, CommonRequestDTO requestDTO) {
        List<DefineTypeRel> floorDefineTypeRel = getFloorDefineTypeRel();
        Set<String> bimTypeIds = floorDefineTypeRel.stream().filter(k->StringUtils.isNotEmpty(k.getBimTypeId())).map(k->k.getBimTypeId()).collect(Collectors.toSet());
        QueryCriteria queryCriteria = new QueryCriteria();
        return null;
    }

    /**
     * 查询楼层列表
     * @param context
     * @param requestDTO
     * @return
     */
    private List<JSONObject> queryFloorList(InstanceUrlParam context, CommonRequestDTO requestDTO){
        // 条件转换
        QueryCriteria queryCriteria = EntityTransferUtil.transDmpRelationCriteria(RelationTypeEnum.ARCHSUBSET_BD2FL.getGraphCode(), RelationTypeEnum.ARCHSUBSET_BD2FL.getRelCode(), requestDTO.getBuildId(), 0);
        // 查询建筑下的楼层列表
        CommonResult response = baseService.doQuery(context,queryCriteria ,null);
        return (List<JSONObject>)response.getContent();
    }

    /**
     * 获取各楼层总面积
     * @param floorList
     * @param key
     * @return
     */
    private Long getFloorArea(List<JSONObject> floorList,String key){
        Set<Long> areas = floorList.stream().filter(k->k.getLong(key) !=null).map(k->k.getLong(key)).collect(Collectors.toSet());
        if(CollUtil.isEmpty(areas)){
            return 0L;
        }
        Long sum = areas.stream().reduce(0L, (current, number) -> current + number);
        return sum;
    }

    /**
     * 查询楼层模块下的空间/设备/阀门类型
     */
    private List<DefineTypeRel> getFloorDefineTypeRel(){
        QueryWrapper<DefineTypeRel> wrapper = new QueryWrapper<>();
        List<DefineTypeRel> defineTypeRels = defineTypeRelMapper.selectList(wrapper);
        if(CollUtil.isEmpty(defineTypeRels)){
            return new ArrayList<>();
        }
        //过滤楼层中不需要的类型
        Iterator<DefineTypeRel> iterator = defineTypeRels.iterator();
        while (iterator.hasNext()){
            DefineTypeRel rel = iterator.next();
            if(StringUtils.isNotEmpty(rel.getModelCode()) && !"LC".equals(rel.getModelCode())){
                iterator.remove();
            }
        }
        return defineTypeRels;
    }

}
